#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location=0) in vec3 position;
layout(location=1) in vec3 normal;
layout(location=2) in vec3 tangent;
layout(location=3) in vec3 color;
layout(location=4) in vec2 UV;

// Output data ; will be interpolated for each fragment.
out vec2 varyingUV;
out vec3 Normal_worldspace;
out vec3 EyeDirection_cameraspace;
out vec4 ShadowCoord;
out vec3 tangentout;

// Values that stay constant for the whole mesh.
uniform mat4 DepthBiasMVP;
uniform mat4 MVP;
uniform mat4 MV;
uniform mat4 M;

void main(){

	// Output position of the vertex, in clip space : MVP * position
	gl_Position =  MVP * vec4(position,1);
	
	ShadowCoord = DepthBiasMVP * vec4(position,1);
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	EyeDirection_cameraspace = -normalize(( MV * vec4(position,1)).xyz);

	Normal_worldspace = normalize(mat3(M)*normal);
	tangentout = (M*vec4(tangent,0.0)).xyz;

	// UV of the vertex. No special space for this one.
	varyingUV = UV;
}

